@@ -1,4 +1,4 @@
-subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar, &
+subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar,&
                      hessian,gradient,value,errorflag)
 !--------------------------------------------------------------------------------------------------
 ! subroutine fdhessian is used to calculate the gradient, hessian matrix of a 
@@ -15,15 +15,17 @@ subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar, &
 ! hessian(npars,npars) :: output, real values, the hessian matrix
 ! gradient(npars)      :: output, real values, the gradient of the parameters
 ! value                :: output, real value, the correspond function value for the specified parameters 
-! errorflag(3)         :: output, integer values, if value can be calculated, errorflag(1)=0, otherwise 1;
-!                         if gradient can be calculated, errorflag(2)=0, otherwise 1; if hessian can be 
-!                         calculated, errorflag(3)=0, otherwise 1. but if sigular matrix appears when 
-!                         attempt to call subroutine GJordan to approximate value, gradient and hessian, 
-!                         all values in errorflag will be 1
+! errorflag(3)         :: output, integer values, error message :
+!                         1) if value can be calculated, errorflag(1)=0, otherwise 1;
+!                         2) if gradient can be calculated, errorflag(2)=0, otherwise 1; 
+!                         3) if hessian can be calculated, errorflag(3)=0, otherwise 1; 
+!                         4) if any error appears in dynamic array allocation, errorflag(4) will be 1, else 0.  
+!                            but if sigular matrix appears when attempt to call subroutine GJordan to approximate 
+!                            value, gradient and hessian, all values in errorflag will be 1
 !
 ! Dependency:: subroutine GJordan, inter function fun34
 !
-! Author:: Peng Jun, 2013.01.27, revised in 2013.03.17, revised again in 2013.04.21
+! Author:: Peng Jun, 2013.01.27, revised in 2013.03.17, revised again in 2013.04.21, last revised in 2013.06.09
 !
 ! Reference:  Jose Pinheiro, Douglas Bates, Saikat DebRoy, Deepayan Sarkar and the
 !             R Development Core Team (2013). nlme: Linear and Nonlinear Mixed
@@ -34,7 +36,7 @@ subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar, &
   implicit none
   integer(kind=4),intent(in)::npars                 
   integer(kind=4),intent(in)::nED                    
-  integer(kind=4),intent(out)::errorflag(3)             
+  integer(kind=4),intent(out)::errorflag(4)             
   real   (kind=8),intent(in)::pars(npars)            
   real   (kind=8),intent(in)::ED(nED)                
   real   (kind=8),intent(in)::Error(nED)            
@@ -47,17 +49,16 @@ subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar, &
   integer(kind=4)::i,j,p
   integer(kind=4)::ncols
   integer(kind=4)::solerror
-  real  (kind=8)::incr(npars)
-  real  (kind=8)::diagpar(npars,npars)
-  real  (kind=8),allocatable::frac(:),cfrac(:)
-  real  (kind=8)::ffrac(1+2*npars)
-  real  (kind=8),allocatable::cols(:,:),ccols(:,:),shifted(:,:),transcols(:,:)
-  real  (kind=8)::pcols(npars,2*npars+1)
-  real  (kind=8),allocatable::xcols(:,:),cxcols(:,:)
-  real  (kind=8),allocatable::pxcols(:,:)
-  real(kind=8),parameter::eps=2.013409D-05 ![.Machine$double.eps^0.3 in R]                    
+  real   (kind=8)::incr(npars)
+  real   (kind=8)::diagpar(npars,npars)
+  real   (kind=8),allocatable::frac(:),cfrac(:)
+  real   (kind=8)::ffrac(1+2*npars)
+  real   (kind=8),allocatable::cols(:,:),ccols(:,:),shifted(:,:),transcols(:,:)
+  real   (kind=8)::pcols(npars,2*npars+1)
+  real   (kind=8),allocatable::xcols(:,:),cxcols(:,:)
+  real   (kind=8),allocatable::pxcols(:,:)
+  real   (kind=8),parameter::eps=2.013409D-05 ![.Machine$double.eps^0.3 in R]                 
   !
-  errorflag=0
   ! Decide the incr values
   ! for each initial par in pars, check if it
   ! is smaller than minabspar, the incr will
@@ -86,21 +87,32 @@ subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar, &
   pcols(:,1)=0.0D+00
   pcols(:,2:npars+1)=diagpar
   pcols(:,npars+2:2*npars+1)=-diagpar
+  !
+  ! default return values if error appears
+  value=-99.0D+00
+  gradient=-99.0D+00
+  hessian=-99.0D+00
+  errorflag=0
+  !
   ! in a total of (npars-1) times looping
   ! add  (npars-i) columns to cols in each loop number
   ! hence a toal of npars*(npars-1)/2 columns for cols
-  allocate( cols(1:npars,1:npars*(npars-1)/2) )
+  allocate( cols(1:npars,1:npars*(npars-1)/2), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   ! in a total of (npars-1) times looping
   ! add (npars-i) number to frac in each loop number
   ! hence a total of npars*(npars-1)/2 values for frac
-  allocate( frac(1:npars*(npars-1)/2) )
+  allocate( frac(1:npars*(npars-1)/2), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   !
   ncols=0
   do i=1,npars-1
     ! in each loop(i), generate ccols and cfrac with 
     ! npars rows (npars-i) columns 
-    allocate( ccols(1:npars,1:npars-i) )
-    allocate( cfrac(1:npars-i) )
+    allocate( ccols(1:npars,1:npars-i), stat=errorflag(4))
+    if( errorflag(4)/=0 ) return
+    allocate( cfrac(1:npars-i), stat=errorflag(4))
+    if( errorflag(4)/=0 ) return
     ! fill ccols and cfrac in each loop
     do j=i+1,npars
       ccols(:,j-i)=diagpar(:,i)+diagpar(:,j)
@@ -120,24 +132,28 @@ subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar, &
   end do
   ! now add up pcols and cols togeteher to be ccols
   ! ccols has a total of ( 2*npars+1 + npars*(npars-1)/2 ) columns
-  allocate( ccols(1:npars,1:npars*(npars-1)/2+2*npars+1) )
+  allocate( ccols(1:npars,1:npars*(npars-1)/2+2*npars+1), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   ccols(:,1:2*npars+1)=pcols
   ccols(:,2*npars+2:npars*(npars-1)/2+2*npars+1)=cols
   ! not need cols presently, so release it
   deallocate(cols)
   ! allocate cols again to store ccols, and release ccols
-  allocate(cols(1:npars,1:npars*(npars-1)/2+2*npars+1))
+  allocate(cols(1:npars,1:npars*(npars-1)/2+2*npars+1), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   cols=ccols
   deallocate(ccols)
   ! now add up ffrac and frac together to be cfrac,
   ! cfrac has a total of ( 2*npars+1 + npars*(npars-1)/2 ) values
-  allocate( cfrac(1:npars*(npars-1)/2+2*npars+1) )
+  allocate( cfrac(1:npars*(npars-1)/2+2*npars+1) ,stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   cfrac(1:2*npars+1)=ffrac
   cfrac(2*npars+2:npars*(npars-1)/2+2*npars+1)=frac
   ! not need frac so release it
   deallocate(frac)
   ! allocate frac again, store cfrac in it then release cfrac
-  allocate(frac(1:npars*(npars-1)/2+2*npars+1))
+  allocate(frac(1:npars*(npars-1)/2+2*npars+1), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   frac=cfrac
   deallocate(cfrac) 
   ! specify p to be the column number of cols
@@ -145,19 +161,22 @@ subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar, &
   p=npars*(npars-1)/2+2*npars+1
   ! now allocate shifted to be the same shape 
   ! with cols and store some values in it
-  allocate(shifted(1:npars,p))
+  allocate(shifted(1:npars,p), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   do i=1,p
     shifted(:,i)=pars+incr*cols(:,i)
   end do
   ! allocate transcols to store transposed cols
   ! transcols has p rows, npars columns
-  allocate( transcols(1:p,1:npars) )
+  allocate( transcols(1:p,1:npars) ,stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   transcols=transpose(cols)
   ! now cols will not be needed, release it
   deallocate(cols)
   ! allocate pxcols to store transcols 
   ! in differ style
-  allocate(pxcols(p,1+2*npars))
+  allocate(pxcols(p,1+2*npars), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   pxcols(:,1)=1.0D+00
   pxcols(:,2:npars+1)=transcols
   pxcols(:,npars+2:2*npars+1)=(transcols)**2
@@ -165,11 +184,13 @@ subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar, &
   ncols=0
   ! now allocate xcols with p rows and 
   ! npars*(npars-1)/2 columns
-  allocate( xcols(p,1:npars*(npars-1)/2) )
+  allocate( xcols(p,1:npars*(npars-1)/2), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   ! 
   do i=1,npars-1
     ! in each loop, allocate cxcols 
-    allocate(cxcols(p,1:npars-i))
+    allocate(cxcols(p,1:npars-i), stat=errorflag(4))
+    if( errorflag(4)/=0 ) return
     do j=i+1,npars
       cxcols(:,j-i)=transcols(:,i)*transcols(:,j)
     end do
@@ -180,21 +201,24 @@ subroutine fdhessian(pars,ED,Error,npars,nED,tol,minAbsPar, &
     ncols=ncols+npars-i  
   end do
   ! now store pxcols and xcols together to cxcols
-  allocate( cxcols(p,1:1+2*npars+npars*(npars-1)/2) )
+  allocate( cxcols(p,1:1+2*npars+npars*(npars-1)/2), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   cxcols(:,1:1+2*npars)=pxcols
   cxcols(:,2+2*npars:1+2*npars+npars*(npars-1)/2)=xcols
   ! release xcols and pxcols
   deallocate(xcols)
   deallocate(pxcols)
   ! allocate xcols again to store cxcols
-  allocate(xcols(p,1:1+2*npars+npars*(npars-1)/2))
+  allocate(xcols(p,1:1+2*npars+npars*(npars-1)/2), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   xcols=cxcols
   ! release cxcols
   deallocate(cxcols)
   ! allocate pxcols again and store some
   ! values to it then release shifted
   ! now pxcols has p rows and 1 column
-  allocate(pxcols(1:p,1))
+  allocate(pxcols(1:p,1), stat=errorflag(4))
+  if( errorflag(4)/=0 ) return
   do i=1,p
     pxcols(i,:)=fun34(shifted(:,i))
   end do
